/**
 * Table metadata is reported by items submitted to the data mapper via methods
 * identified by symbols. This is done both to disambiguate data (which should
 * always be identified by string keys) from metadata and also to allow an
 * eventually integration with the First-Class Protocols proposal as described
 * at {@link https://github.com/michaelficarra/proposal-first-class-protocols}
 * (currently at stage 1 in the ECMAScript change acceptance process).
 *
 * Because the protocol proposal allows implementation to be declared
 * dynamically at runtime (and also because TypeScript does not allow
 * user-defined symbols to appear in type declarations), protocol adherence
 * should be detected on objects at runtime rather than on types via static
 * analysis.
 */

export const DocumentSchema = Symbol('DocumentSchema');

export function getSchema(item: any): any {
    if (item) {
        const schema = item[DocumentSchema];
        if (schema && typeof schema === 'object') {
            return schema;
        }
    }

    throw new Error(
        'The provided item did not adhere to the Document protocol.' +
        ' No object property was found at the `DocumentSchema` symbol'
    );
}

export const METADATA_TYPE_KEY = 'design:type';
