import { SchemaType, ZeroArgumentsConstructor } from './schema-type';
import { DocumentSchema } from './protocols';

export function unmarshallItem<T = { [key: string]: any }>(
  input: any,
  valueConstructor: ZeroArgumentsConstructor<T>,
): T {
  const unmarshalled: T = new valueConstructor();

  const schema = unmarshalled[DocumentSchema];

  for (const key of Object.keys(schema)) {
    const { attributeName = key } = schema[key];
    if (attributeName in input) {
      (unmarshalled as { [key: string]: any })[key] = unmarshallValue(
        schema[key],
        input[attributeName],
      );
    }
  }

  return unmarshalled;
}

function unmarshallValue(schemaType: SchemaType, input: any): any {
  switch (schemaType.type) {
    case 'String':
      return input;
    case 'Boolean':
      if (input === 1) {
        return true;
      } else if (input === 0) {
        return false;
      }
      return input;
    case 'Document':
      return input ? unmarshallItem(input, schemaType.valueConstructor) : undefined;
    case 'Collection':
      return input ? input.map(item => {
        if (schemaType.documentConstructorMapping) {
          const attributeValue = item[schemaType.documentConstructorMapping.attribute];
          const valueConstructor = schemaType.documentConstructorMapping.mapping[attributeValue];
          if (valueConstructor) {
            return unmarshallItem(item, valueConstructor)
          }
        }
        return unmarshallItem(item, schemaType.valueConstructor)
      }) : undefined;
  }
  return input;
}
